/***************************************************************************
                          leaseparser.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>
#include <qtextstream.h>
#include <klocale.h>
#include <kmessagebox.h>
#include "leaseparser.h"
#include "mlistviewitem.h"
#include "server.h"
#include "items.h"

extern int lease_yyparse(void *);
extern int lease_yydebug;
extern int lease_yypos;
extern int lease_lineno;
extern const char *lease_lexinput;

LeaseParser::LeaseParser(QString fileName, MListViewItem *serverItem)
{
	thisFileName = fileName;
	thisServerItem = serverItem;
}

bool LeaseParser::Parse()
{
	if ( KIO::NetAccess::download(thisFileName, tmpFileName) )
	{
		//qDebug(tmpFileName.latin1());
		QString result;
		leasesItem *listLeases;
		QFile configFile( tmpFileName );

		if (configFile.open( IO_ReadOnly ))
		{
			QTextStream ts(&configFile);
			ts.setEncoding(QTextStream::Latin1); // Locale??
			configData = ts.read();
			configFile.close();
			lease_lexinput = configData.ascii();
			lease_yydebug = 0;
			lease_yypos = 0;
			lease_lineno = 1;
			lease_yyparse( this );
			//result = generate();
			//printf("%s", result.ascii());
			listLeases = new leasesItem(thisServerItem, i18n("Leases"), &leases, 0);
			return(true);
		} else {
			return(false);
		}
	}
	else
	{
		KMessageBox::sorry(0, i18n("Unable to read: ") + thisFileName);
		return(false);
	}
}

void LeaseParser::addSyntaxError(int lineno)
{
	int *x = new int(lineno);
	syntaxErrors.append( x );
}

void LeaseParser::addLease(leaseItem *lease)
{
	leases.append(lease);
}

QString LeaseParser::generate()
{
	QString result;
	dataItem *element;

	for ( element = leases.first(); element; element = leases.next() )
		result += element->generate("");
	return result;
}


LeaseParser::~LeaseParser()
{
	leases.clear();
}
