/***************************************************************************
                          server.cpp  -  description
                             -------------------
    copyright            : (C) 2003 by Willy De la Court
    email                : willy.delacourt@pandora.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2 as     *
 *   published by the Free Software Foundation                             *
 *                                                                         *
 ***************************************************************************/

#include <kurl.h>
#include <kmessagebox.h>
#include <qtextedit.h>
#include <qregexp.h>
#include <qheader.h>
#include <kdeversion.h>
#include "msgdialog_ui.h"
#include "syntaxerror_ui.h"
#include "server.h"
#include "confparser.h"
#include "leaseparser.h"
#include "mlistviewitem.h"
#include "syntaxerror.h"
 
addServer::addServer( KConfig* kcmdhcp_config, QListView *thisList, QWidget *parent, const char *name )
	: addServer_UI(parent, name)
{
	configListView = thisList;
	config = kcmdhcp_config;
	configGroup = "0";
	description = i18n("Add a new DHCP server to manage");
	allServersItem = new MListViewItem(configListView, i18n("Servers"), this);
	allServersItem->setOpen(true);
	configListView->setSelected(allServersItem, true);

	connect( btnAdd, SIGNAL( clicked() ), SLOT( add() ) );
}

void addServer::add()
{
	editServer *newServer = new editServer(config, configListView, configGroup, 0, "editServer");
	newServer->serverName->setText( serverName->text() );
	newServer->configFileName->setText( configFileName->text().isEmpty() ? QString("/etc/dhcpd.conf") : configFileName->text() );
	newServer->leasesFileName->setText( leasesFileName->text().isEmpty() ? QString("/var/lib/dhcp/dhcpd.leases") : leasesFileName->text() );
	newServer->serverIsRemote->setChecked( serverIsRemote->isChecked() );
	newServer->remoteHostName->setText( remoteHostName->text().isEmpty() ? QString("localhost") : remoteHostName->text() );
	newServer->SSHPort->setText( SSHPort->text().isEmpty() ? QString("22") : SSHPort->text() );
	newServer->UserName->setText( UserName->text().isEmpty() ? QString("root") : UserName->text() );
	configListView->setSelected( newServer->serverItem, true );
	emit changed( true );
}

editServer::editServer( KConfig *kcmdhcp_config, QListView *thisList, QString serverNumber, QWidget *parent, const char *name )
	: editServer_UI(parent, name)
{
	configListView = thisList;
	allServersItem = configListView->firstChild();
	config = kcmdhcp_config;
	configGroup = serverNumber;
	description = i18n("Edit DHCP server configuration");
	readConfig();
	serverItem = new MListViewItem(allServersItem, serverName->text(), this);
	connect( btnReload, SIGNAL( clicked() ), SLOT( reload() ) );
	connect( btnDelete, SIGNAL( clicked() ), SLOT( deletethis() ) );
	connect( serverName, SIGNAL( textChanged(const QString &) ), SLOT( serverNameChanged(const QString &) ) );
	connect( btnStartServer, SIGNAL( clicked() ), SLOT( actionServerStart() ) );
	connect( btnStopServer, SIGNAL( clicked() ), SLOT( actionServerStop() ) );
	connect( btnRestartServer, SIGNAL( clicked() ), SLOT( actionServerRestart() ) );
	connect( btnReloadServer, SIGNAL( clicked() ), SLOT( actionServerReload() ) );
	connect( btnShowLog, SIGNAL( clicked() ), SLOT( slotShowLog() ) );
	connect( btnShowErrors, SIGNAL( clicked() ), SLOT( slotShowErrors() ) );
	confParser = 0;
	leaseParser = 0;
	configRead = false;
}

void editServer::actionServerStart()   { execute( serverStart->text() );   }
void editServer::actionServerStop()    { execute( serverStop->text() );    }
void editServer::actionServerRestart() { execute( serverRestart->text() ); }
void editServer::actionServerReload()  { execute( serverReload->text() );  }

void editServer::readConfig()
{
	config->setGroup(QString::fromLatin1("Server") + configGroup);

	serverName->setText(config->readEntry("ServerName"));
	configFileName->setText(config->readEntry("Config", "/etc/dhcpd.conf"));
	leasesFileName->setText(config->readEntry("Leases", "/var/lib/dhcp/dhcpd.leases"));
	serverIsRemote->setChecked(config->readBoolEntry("Remote", false));
	remoteHostName->setText(config->readEntry("HostName", "localhost"));
	SSHPort->setText(config->readEntry("SSHPort", "22"));
	UserName->setText(config->readEntry("UserName", "root"));
	serverStop->setText(config->readEntry("ServerStop", "service dhcpd stop"));
	serverStart->setText(config->readEntry("ServerStart", "service dhcpd start"));
	serverRestart->setText(config->readEntry("ServerRestart", "service dhcpd restart"));
	serverReload->setText(config->readEntry("ServerReload", "service dhcpd reload"));
}

void editServer::saveConfig()
{
	if ( configGroup.toInt() == 0 )
	{
		config->setGroup("General");
		QStringList serverList = config->readListEntry("Servers");

		int val = 0;
		for ( QStringList::Iterator it = serverList.begin(); it != serverList.end(); ++it )
		{
			val = ((*it).toInt() > val)?(*it).toInt():val;
		}
		val++;
		configGroup.setNum( val );
		serverList << configGroup;
		config->writeEntry("Servers", serverList);
		config->sync();
	}
	config->setGroup(QString::fromLatin1("Server") + configGroup);
	config->writeEntry( "ServerName", serverName->text() );
	config->writeEntry( "Config", configFileName->text() );
	config->writeEntry( "Leases", leasesFileName->text() );
	config->writeEntry( "Remote", serverIsRemote->isChecked() );
	config->writeEntry( "HostName", remoteHostName->text() );
	config->writeEntry( "SSHPort", SSHPort->text() );
	config->writeEntry( "UserName", UserName->text() );
	config->writeEntry( "ServerStop", serverStop->text());
	config->writeEntry( "ServerStart", serverStart->text());
	config->writeEntry( "ServerRestart", serverRestart->text());
	config->writeEntry( "ServerReload", serverReload->text());
	config->sync();
	saveServerConfig();
}

void editServer::saveServerConfig()
{
	if ( confParser )
		if( confParser->isChanged() )
		{
			qDebug("Changed");
			confParser->save();
		}
}

void editServer::readServerConfig(MListViewItem *thisServerItem)
{
	if ( !configRead )
	{
		confParser = new ConfParser( makeRemoteName( configFileName->text() ), thisServerItem );
		configRead = confParser->Parse();
		if( !confParser->syntaxErrors.isEmpty() )
		{
			int cnt = confParser->syntaxErrors.count();
			KMessageBox::error(0, i18n("A number of errors (%1) where detected when loading the configuration file. Use the 'Show Errors' button to review.").arg( cnt ) );
			btnShowErrors->setEnabled( true );
		}
		else
			btnShowErrors->setEnabled( false );

		leaseParser = new LeaseParser( makeRemoteName( leasesFileName->text() ), thisServerItem );
		configRead |= leaseParser->Parse();
		// open all items under the selected item
		QListViewItem* item = thisServerItem->firstChild();
		for ( ; item ; item = item->itemBelow() )
			item->setOpen( true );
	}
}

void editServer::reload()
{
	if ( configRead )
	{
		while ( serverItem->firstChild() )
		{
			delete serverItem->firstChild();
		}
		delete confParser;
		delete leaseParser;
		configRead = false;
	}
	readServerConfig(serverItem);
}

void editServer::deletethis()
{
	if ( configGroup != "0" )
	{
		config->setGroup("General");
		QStringList serverList = config->readListEntry("Servers");
		QStringList newServerList;
		for ( QStringList::Iterator it = serverList.begin(); it != serverList.end(); ++it )
		{
			if ( (*it) != configGroup )
				newServerList << (*it);
		}
		config->writeEntry("Servers", newServerList);
		// Do not sync yet User can still press cancel
	}
	configListView->setSelected( allServersItem, true );
	emit changed( true );
	delete serverItem;
}

void editServer::serverNameChanged(const QString &newName)
{
	serverItem->setText(0, newName);
}

void editServer::execute(QString command)
{
#if KDE_IS_VERSION(3, 1, 90)
	QString result;
	result = KIO::NetAccess::fish_execute( makeRemoteName( "" ), command, this );
	QRegExp rx("###RESULT: ([0-9]+)");
	rx.search( result );
	int ret = rx.cap( 1 ).toInt();
	resultLog += result.prepend( i18n( "Executing '%1'\n\n" ).arg( command ) ) + "\n";
	result.replace( rx, "");
	// stupid hack add 512 spaces to force the MessageBox to display an editbox.
	if( ret != 0 )
		KMessageBox::detailedError(0, i18n("Execution of command '%1' failed.\n Return code is '%2'.").arg( command ).arg( ret ), result + QString().fill(' ', 512) );
#else
	KMessageBox::sorry(0, i18n("Feature only implemented in KDE version %1.").arg( "3.1.90 (CVS >= 20030225)" ) );
#endif
}

QString editServer::makeRemoteName(QString fileName)
{
	QString remoteFilename;

#if KDE_IS_VERSION(3, 1, 90)
	if ( serverIsRemote->isChecked() )
	{
#endif
		remoteFilename = "fish://";
		remoteFilename += UserName->text() + "@";
		remoteFilename += remoteHostName->text() + ":" + SSHPort->text() ;
		remoteFilename += fileName;
#if KDE_IS_VERSION(3, 1, 90)
	}
	else
	{
		// using the special fish feature for localhost which uses su to gain root access
		remoteFilename = "fish://";
		remoteFilename += "root@localhost";
		remoteFilename += fileName;
	}
#endif
	return remoteFilename;
}

void editServer::slotShowErrors()
{
	if( !confParser->syntaxErrors.isEmpty() )
	{
		syntaxError *s = new syntaxError( confParser->configData, confParser->syntaxErrors, this);
		delete s;
	}
}

void editServer::slotShowLog()
{
	msgDialog_UI *s = new msgDialog_UI(this, "msgDialog", true);
	s->txtMsg->setText(resultLog);
	connect( s->btnClear, SIGNAL( clicked() ), this, SLOT( slotClearLog() ) );
	s->exec();
	delete s;
}

void editServer::slotClearLog()
{
	resultLog = "";
}

#include "server.moc"
